@@ -25,7 +25,7 @@ use crate::{
 };
 use arrow_buffer::bit_util::set_bit;
 use arrow_buffer::buffer::NullBuffer;
-use arrow_buffer::{ArrowNativeType, BooleanBuffer, BooleanBufferBuilder};
+use arrow_buffer::{ArrowNativeType, BooleanBuffer, BooleanBufferBuilder, ScalarBuffer};
 use arrow_data::ArrayData;
 use arrow_schema::{ArrowError, DataType};
 use std::any::Any;
@@ -292,17 +292,20 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
             Box::new(values.data_type().clone()),
         );
 
-        let zero = K::Native::usize_as(0);
-        let values_len = values.len();
+        // // we can skip the validating the keys if they are all null
+        let all_null = keys.null_count() == keys.len();
 
-        if let Some((idx, v)) =
-            keys.values().iter().enumerate().find(|(idx, v)| {
+        if !all_null {
+            let zero = K::Native::usize_as(0);
+            let values_len = values.len();
+
+            if let Some((idx, v)) = keys.values().iter().enumerate().find(|(idx, v)| {
                 (v.is_lt(zero) || v.as_usize() >= values_len) && keys.is_valid(*idx)
-            })
-        {
-            return Err(ArrowError::InvalidArgumentError(format!(
-                "Invalid dictionary key {v:?} at index {idx}, expected 0 <= key < {values_len}",
-            )));
+            }) {
+                return Err(ArrowError::InvalidArgumentError(format!(
+                    "Invalid dictionary key {v:?} at index {idx}, expected 0 <= key < {values_len}",
+                )));
+            }
         }
 
         Ok(Self {
@@ -580,21 +583,25 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
     }
 }
 
-/// Constructs a `DictionaryArray` from an array data reference.
+/// Constructs a `DictionaryArray` from an `ArrayData`
 impl<T: ArrowDictionaryKeyType> From<ArrayData> for DictionaryArray<T> {
     fn from(data: ArrayData) -> Self {
+        let (data_type, len, nulls, offset, mut buffers, mut child_data) = data.into_parts();
+
         assert_eq!(
-            data.buffers().len(),
+            buffers.len(),
             1,
             "DictionaryArray data should contain a single buffer only (keys)."
         );
+        let buffer = buffers.pop().expect("checked above");
         assert_eq!(
-            data.child_data().len(),
+            child_data.len(),
             1,
             "DictionaryArray should contain a single child array (values)."
         );
+        let cd = child_data.pop().expect("checked above");
 
-        if let DataType::Dictionary(key_data_type, _) = data.data_type() {
+        if let DataType::Dictionary(key_data_type, _) = &data_type {
             assert_eq!(
                 &T::DATA_TYPE,
                 key_data_type.as_ref(),
@@ -603,19 +610,10 @@ impl<T: ArrowDictionaryKeyType> From<ArrayData> for DictionaryArray<T> {
                 key_data_type
             );
 
-            let values = make_array(data.child_data()[0].clone());
-            let data_type = data.data_type().clone();
+            let values = make_array(cd);
 
             // create a zero-copy of the keys' data
-            // SAFETY:
-            // ArrayData is valid and verified type above
-
-            let keys = PrimitiveArray::<T>::from(unsafe {
-                data.into_builder()
-                    .data_type(T::DATA_TYPE)
-                    .child_data(vec![])
-                    .build_unchecked()
-            });
+            let keys = PrimitiveArray::<T>::new(ScalarBuffer::new(buffer, offset, len), nulls);
 
             Self {
                 data_type,
@@ -1052,7 +1050,7 @@ impl<K: ArrowDictionaryKeyType> AnyDictionaryArray for DictionaryArray<K> {
 mod tests {
     use super::*;
     use crate::cast::as_dictionary_array;
-    use crate::{Int8Array, Int16Array, Int32Array, RunArray};
+    use crate::{Int8Array, Int16Array, Int32Array, RunArray, UInt8Array};
     use arrow_buffer::{Buffer, ToByteSlice};
 
     #[test]
@@ -1533,4 +1531,13 @@ mod tests {
         let dictionary = DictionaryArray::new(keys, Arc::new(Int32Array::new_null(2)));
         assert_eq!(&dictionary.normalized_keys(), &[1, 0, 1])
     }
+
+    #[test]
+    fn test_all_null_dict() {
+        let all_null_dict_arr = DictionaryArray::try_new(
+            UInt8Array::new_null(10),
+            Arc::new(StringArray::from_iter_values(["a"])),
+        );
+        assert!(all_null_dict_arr.is_ok())
+    }
 }
