@@ -15,8 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::array::{get_offsets, make_array, print_long_array};
-use crate::builder::{GenericListBuilder, PrimitiveBuilder};
+use crate::array::{get_offsets_from_buffer, make_array, print_long_array};
+use crate::builder::{ArrayBuilder, GenericListBuilder, PrimitiveBuilder};
 use crate::{
     Array, ArrayAccessor, ArrayRef, ArrowPrimitiveType, FixedSizeListArray,
     iterator::GenericListArrayIter, new_empty_array,
@@ -418,18 +418,43 @@ impl<OffsetSize: OffsetSizeTrait> GenericListArray<OffsetSize> {
         T: ArrowPrimitiveType,
         P: IntoIterator<Item = Option<<T as ArrowPrimitiveType>::Native>>,
         I: IntoIterator<Item = Option<P>>,
+    {
+        Self::from_nested_iter::<PrimitiveBuilder<T>, T::Native, P, I>(iter)
+    }
+
+    /// Creates a [`GenericListArray`] from a nested iterator of values.
+    /// This method works for any values type that has a corresponding builder that implements the
+    /// `Extend` trait. That includes all numeric types, booleans, binary and string types and also
+    /// dictionary encoded binary and strings.
+    ///
+    /// # Example
+    /// ```
+    /// # use arrow_array::ListArray;
+    /// # use arrow_array::types::Int32Type;
+    /// # use arrow_array::builder::StringDictionaryBuilder;
+    /// let data = vec![
+    ///    Some(vec![Some("foo"), Some("bar"), Some("baz")]),
+    ///    None,
+    ///    Some(vec![Some("bar"), None, Some("foo")]),
+    ///    Some(vec![]),
+    /// ];
+    /// let list_array = ListArray::from_nested_iter::<StringDictionaryBuilder<Int32Type>, _, _, _>(data);
+    /// println!("{:?}", list_array);
+    /// ```
+    pub fn from_nested_iter<B, T, P, I>(iter: I) -> Self
+    where
+        B: ArrayBuilder + Default + Extend<Option<T>>,
+        P: IntoIterator<Item = Option<T>>,
+        I: IntoIterator<Item = Option<P>>,
     {
         let iter = iter.into_iter();
         let size_hint = iter.size_hint().0;
-        let mut builder =
-            GenericListBuilder::with_capacity(PrimitiveBuilder::<T>::new(), size_hint);
+        let mut builder = GenericListBuilder::with_capacity(B::default(), size_hint);
 
         for i in iter {
             match i {
                 Some(p) => {
-                    for t in p {
-                        builder.values().append_option(t);
-                    }
+                    builder.values().extend(p);
                     builder.append(true);
                 }
                 None => builder.append(false),
@@ -479,23 +504,26 @@ impl<OffsetSize: OffsetSizeTrait> From<FixedSizeListArray> for GenericListArray<
 
 impl<OffsetSize: OffsetSizeTrait> GenericListArray<OffsetSize> {
     fn try_new_from_array_data(data: ArrayData) -> Result<Self, ArrowError> {
-        if data.buffers().len() != 1 {
+        let (data_type, len, nulls, offset, mut buffers, mut child_data) = data.into_parts();
+
+        if buffers.len() != 1 {
             return Err(ArrowError::InvalidArgumentError(format!(
                 "ListArray data should contain a single buffer only (value offsets), had {}",
-                data.buffers().len()
+                buffers.len()
             )));
         }
+        let buffer = buffers.pop().expect("checked above");
 
-        if data.child_data().len() != 1 {
+        if child_data.len() != 1 {
             return Err(ArrowError::InvalidArgumentError(format!(
                 "ListArray should contain a single child array (values array), had {}",
-                data.child_data().len()
+                child_data.len()
             )));
         }
 
-        let values = data.child_data()[0].clone();
+        let values = child_data.pop().expect("checked above");
 
-        if let Some(child_data_type) = Self::get_type(data.data_type()) {
+        if let Some(child_data_type) = Self::get_type(&data_type) {
             if values.data_type() != child_data_type {
                 return Err(ArrowError::InvalidArgumentError(format!(
                     "[Large]ListArray's child datatype {:?} does not \
@@ -506,19 +534,18 @@ impl<OffsetSize: OffsetSizeTrait> GenericListArray<OffsetSize> {
             }
         } else {
             return Err(ArrowError::InvalidArgumentError(format!(
-                "[Large]ListArray's datatype must be [Large]ListArray(). It is {:?}",
-                data.data_type()
+                "[Large]ListArray's datatype must be [Large]ListArray(). It is {data_type:?}",
             )));
         }
 
         let values = make_array(values);
         // SAFETY:
         // ArrayData is valid, and verified type above
-        let value_offsets = unsafe { get_offsets(&data) };
+        let value_offsets = unsafe { get_offsets_from_buffer(buffer, offset, len) };
 
         Ok(Self {
-            data_type: data.data_type().clone(),
-            nulls: data.nulls().cloned(),
+            data_type,
+            nulls,
             values,
             value_offsets,
         })
@@ -632,10 +659,15 @@ pub type LargeListArray = GenericListArray<i64>;
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::builder::{FixedSizeListBuilder, Int32Builder, ListBuilder, UnionBuilder};
+    use crate::builder::{
+        BooleanBuilder, FixedSizeListBuilder, Int32Builder, ListBuilder, StringBuilder,
+        StringDictionaryBuilder, UnionBuilder,
+    };
     use crate::cast::AsArray;
-    use crate::types::Int32Type;
-    use crate::{Int32Array, Int64Array};
+    use crate::types::{Int8Type, Int32Type};
+    use crate::{
+        BooleanArray, Int8Array, Int8DictionaryArray, Int32Array, Int64Array, StringArray,
+    };
     use arrow_buffer::{Buffer, ScalarBuffer, bit_util};
     use arrow_schema::Field;
 
@@ -1292,4 +1324,60 @@ mod tests {
         let array = ListArray::new_null(field, 5);
         assert_eq!(array.len(), 5);
     }
+
+    #[test]
+    fn test_list_from_iter_i32() {
+        let array = ListArray::from_nested_iter::<Int32Builder, _, _, _>(vec![
+            None,
+            Some(vec![Some(1), None, Some(2)]),
+        ]);
+        let expected_offsets = &[0, 0, 3];
+        let expected_values: ArrayRef = Arc::new(Int32Array::from(vec![Some(1), None, Some(2)]));
+        assert_eq!(array.value_offsets(), expected_offsets);
+        assert_eq!(array.values(), &expected_values);
+    }
+
+    #[test]
+    fn test_list_from_iter_bool() {
+        let array = ListArray::from_nested_iter::<BooleanBuilder, _, _, _>(vec![
+            Some(vec![None, Some(false), Some(true)]),
+            None,
+        ]);
+        let expected_offsets = &[0, 3, 3];
+        let expected_values: ArrayRef =
+            Arc::new(BooleanArray::from(vec![None, Some(false), Some(true)]));
+        assert_eq!(array.value_offsets(), expected_offsets);
+        assert_eq!(array.values(), &expected_values);
+    }
+
+    #[test]
+    fn test_list_from_iter_str() {
+        let array = ListArray::from_nested_iter::<StringBuilder, _, _, _>(vec![
+            Some(vec![Some("foo"), None, Some("bar")]),
+            None,
+        ]);
+        let expected_offsets = &[0, 3, 3];
+        let expected_values: ArrayRef =
+            Arc::new(StringArray::from(vec![Some("foo"), None, Some("bar")]));
+        assert_eq!(array.value_offsets(), expected_offsets);
+        assert_eq!(array.values(), &expected_values);
+    }
+
+    #[test]
+    fn test_list_from_iter_dict_str() {
+        let array =
+            ListArray::from_nested_iter::<StringDictionaryBuilder<Int8Type>, _, _, _>(vec![
+                Some(vec![Some("foo"), None, Some("bar"), Some("foo")]),
+                None,
+            ]);
+        let expected_offsets = &[0, 4, 4];
+        let expected_dict_values: ArrayRef =
+            Arc::new(StringArray::from(vec![Some("foo"), Some("bar")]));
+        let expected_dict_keys = Int8Array::from(vec![Some(0), None, Some(1), Some(0)]);
+        let expected_values: ArrayRef = Arc::new(
+            Int8DictionaryArray::try_new(expected_dict_keys, expected_dict_values).unwrap(),
+        );
+        assert_eq!(array.value_offsets(), expected_offsets);
+        assert_eq!(array.values(), &expected_values);
+    }
 }
