@@ -17,7 +17,7 @@
 
 //! Idiomatic iterator for [`RunArray`](crate::RunArray)
 
-use crate::{array::ArrayAccessor, types::RunEndIndexType, Array, TypedRunArray};
+use crate::{Array, TypedRunArray, array::ArrayAccessor, types::RunEndIndexType};
 use arrow_buffer::ArrowNativeType;
 
 /// The [`RunArrayIter`] provides an idiomatic way to iterate over the run array.
@@ -172,13 +172,13 @@ where
 
 #[cfg(test)]
 mod tests {
-    use rand::{seq::SliceRandom, thread_rng, Rng};
+    use rand::{Rng, rng, seq::SliceRandom};
 
     use crate::{
+        Array, Int64RunArray, PrimitiveArray, RunArray,
         array::{Int32Array, StringArray},
         builder::PrimitiveRunBuilder,
         types::{Int16Type, Int32Type},
-        Array, Int64RunArray, PrimitiveArray, RunArray,
     };
 
     fn build_input_array(size: usize) -> Vec<Option<i32>> {
@@ -200,7 +200,7 @@ mod tests {
         ];
         let mut result: Vec<Option<i32>> = Vec::with_capacity(size);
         let mut ix = 0;
-        let mut rng = thread_rng();
+        let mut rng = rng();
         // run length can go up to 8. Cap the max run length for smaller arrays to size / 2.
         let max_run_length = 8_usize.min(1_usize.max(size / 2));
         while result.len() < size {
@@ -209,7 +209,7 @@ mod tests {
                 seed.shuffle(&mut rng);
             }
             // repeat the items between 1 and 8 times. Cap the length for smaller sized arrays
-            let num = max_run_length.min(rand::thread_rng().gen_range(1..=max_run_length));
+            let num = max_run_length.min(rng.random_range(1..=max_run_length));
             for _ in 0..num {
                 result.push(seed[ix]);
             }
