@@ -20,9 +20,9 @@ use std::fmt::Debug;
 use std::ptr::NonNull;
 use std::sync::Arc;
 
+use crate::BufferBuilder;
 use crate::alloc::{Allocation, Deallocation};
 use crate::util::bit_chunk_iterator::{BitChunks, UnalignedBitChunk};
-use crate::{BooleanBuffer, BufferBuilder};
 use crate::{bit_util, bytes::Bytes, native::ArrowNativeType};
 
 #[cfg(feature = "pool")]
@@ -343,7 +343,17 @@ impl Buffer {
             return self.slice_with_length(offset / 8, bit_util::ceil(len, 8));
         }
 
-        BooleanBuffer::from_bits(self.as_slice(), offset, len).into_inner()
+        let chunks = self.bit_chunks(offset, len);
+
+        let buffer: Vec<u64> = if chunks.remainder_len() > 0 {
+            chunks.iter().chain(Some(chunks.remainder_bits())).collect()
+        } else {
+            chunks.iter().collect()
+        };
+        let mut buffer = Buffer::from_vec(buffer);
+        // Update length to be byte-aligned
+        buffer.length = bit_util::ceil(len, 8);
+        buffer
     }
 
     /// Returns a `BitChunks` instance which can be used to iterate over this buffers bits
