@@ -15,7 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use super::{Extend, _MutableArrayData};
+use super::{_MutableArrayData, Extend};
 use crate::ArrayData;
 
 pub(super) fn build_extend_sparse(array: &ArrayData) -> Extend<'_> {
