@@ -92,11 +92,7 @@ pub fn create_random_array(
         UInt16 => Arc::new(create_primitive_array::<UInt16Type>(size, null_density)),
         UInt32 => Arc::new(create_primitive_array::<UInt32Type>(size, null_density)),
         UInt64 => Arc::new(create_primitive_array::<UInt64Type>(size, null_density)),
-        Float16 => {
-            return Err(ArrowError::NotYetImplemented(
-                "Float16 is not implemented".to_string(),
-            ));
-        }
+        Float16 => Arc::new(create_primitive_array::<Float16Type>(size, null_density)),
         Float32 => Arc::new(create_primitive_array::<Float32Type>(size, null_density)),
         Float64 => Arc::new(create_primitive_array::<Float64Type>(size, null_density)),
         Timestamp(unit, tz) => match unit {
@@ -545,6 +541,7 @@ mod tests {
         let size = 32;
         let fields = vec![
             Field::new("a", DataType::Int32, true),
+            Field::new("f16", DataType::Float16, true),
             Field::new(
                 "timestamp_without_timezone",
                 DataType::Timestamp(TimeUnit::Nanosecond, None),
