@@ -34,5 +34,5 @@
 mod from_json;
 mod to_json;
 
-pub use from_json::JsonToVariant;
+pub use from_json::{JsonToVariant, append_json};
 pub use to_json::VariantToJson;
