@@ -317,6 +317,18 @@ impl ByteViewArrayDecoderPlain {
     }
 
     pub fn read(&mut self, output: &mut ViewBuffer, len: usize) -> Result<usize> {
+        if self.validate_utf8 {
+            self.read_impl::<true>(output, len)
+        } else {
+            self.read_impl::<false>(output, len)
+        }
+    }
+
+    fn read_impl<const VALIDATE_UTF8: bool>(
+        &mut self,
+        output: &mut ViewBuffer,
+        len: usize,
+    ) -> Result<usize> {
         // avoid creating a new buffer if the last buffer is the same as the current buffer
         // This is especially useful when row-level filtering is applied, where we call lots of small `read` over the same buffer.
         let block_id = {
@@ -330,28 +342,37 @@ impl ByteViewArrayDecoderPlain {
         let to_read = len.min(self.max_remaining_values);
 
         let buf: &[u8] = self.buf.as_ref();
-        let mut read = 0;
+        let buf_len = buf.len();
+        let mut end_offset = self.offset;
+        let mut utf8_validation_begin = end_offset;
+
         output.views.reserve(to_read);
 
-        let mut utf8_validation_begin = self.offset;
-        while self.offset < self.buf.len() && read != to_read {
-            if self.offset + 4 > self.buf.len() {
+        // Safety: we reserved enough space in output.views
+        // and we will only write up to to_read views / track how many views we wrote.
+        // Ideally, we would use `Vec::extend` here, but this generates sub-optimal code.
+        let views_ptr = output.views.as_mut_ptr().wrapping_add(output.views.len());
+        for i in 0..to_read {
+            let start_offset = end_offset + 4;
+
+            if start_offset > buf_len {
                 return Err(ParquetError::EOF("eof decoding byte array".into()));
             }
-            let len_bytes: [u8; 4] = unsafe {
-                buf.get_unchecked(self.offset..self.offset + 4)
+
+            // Safety: we have checked that start_offset <= buf_len
+            let len = u32::from_le_bytes(
+                unsafe { buf.get_unchecked(end_offset..start_offset) }
                     .try_into()
-                    .unwrap()
-            };
-            let len = u32::from_le_bytes(len_bytes);
+                    .unwrap(),
+            );
 
-            let start_offset = self.offset + 4;
-            let end_offset = start_offset + len as usize;
-            if end_offset > buf.len() {
+            end_offset = start_offset + len as usize;
+
+            if end_offset > buf_len {
                 return Err(ParquetError::EOF("eof decoding byte array".into()));
             }
 
-            if self.validate_utf8 {
+            if VALIDATE_UTF8 {
                 // It seems you are trying to understand what's going on here, take a breath and be patient.
                 // Utf-8 validation is a non-trivial task, here are some background facts:
                 // (1) Validating one 2048-byte string is much faster than validating 128 of 16-byte string.
@@ -373,32 +394,40 @@ impl ByteViewArrayDecoderPlain {
                 // The implementation keeps a water mark `utf8_validation_begin` to track the beginning of the buffer that is not validated.
                 // If the length is smaller than 128, then we continue to next string.
                 // If the length is larger than 128, then we validate the buffer before the length bytes, and move the water mark to the beginning of next string.
-                if len < 128 {
-                    // fast path, move to next string.
-                    // the len bytes are valid utf8.
-                } else {
+                if len >= 128 {
                     // unfortunately, the len bytes may not be valid utf8, we need to wrap up and validate everything before it.
                     check_valid_utf8(unsafe {
-                        buf.get_unchecked(utf8_validation_begin..self.offset)
+                        buf.get_unchecked(utf8_validation_begin..start_offset - 4)
                     })?;
                     // move the cursor to skip the len bytes.
                     utf8_validation_begin = start_offset;
                 }
             }
 
+            let view = make_view(
+                unsafe { buf.get_unchecked(start_offset..end_offset) },
+                block_id,
+                start_offset as u32,
+            );
+            // Safety: views_ptr is valid for writes, and we have reserved enough space.
             unsafe {
-                output.append_view_unchecked(block_id, start_offset as u32, len);
+                views_ptr.add(i).write(view);
             }
-            self.offset = end_offset;
-            read += 1;
         }
 
-        // validate the last part of the buffer
-        if self.validate_utf8 {
-            check_valid_utf8(unsafe { buf.get_unchecked(utf8_validation_begin..self.offset) })?;
+        // Safety: we have written `to_read` views to `views_ptr`
+        unsafe {
+            output.views.set_len(output.views.len() + to_read);
+        }
+        if VALIDATE_UTF8 {
+            // validate values from the previously validated location up to (but not including)
+            // the length of this string
+            check_valid_utf8(unsafe { buf.get_unchecked(utf8_validation_begin..end_offset) })?;
         }
 
+        self.offset = end_offset;
         self.max_remaining_values -= to_read;
+
         Ok(to_read)
     }
 
@@ -439,6 +468,8 @@ impl ByteViewArrayDecoderDictionary {
     /// Assumptions / Optimization
     /// This function checks if dict.buffers() are the last buffers in `output`, and if so
     /// reuses the dictionary page buffers directly without copying data
+    ///
+    /// If the dictionary is empty, the buffer contains empty view.
     fn read(&mut self, output: &mut ViewBuffer, dict: &ViewBuffer, len: usize) -> Result<usize> {
         if dict.is_empty() || len == 0 {
             return Ok(0);
@@ -469,32 +500,50 @@ impl ByteViewArrayDecoderDictionary {
         // then the base_buffer_idx is 5 - 2 = 3
         let base_buffer_idx = output.buffers.len() as u32 - dict.buffers.len() as u32;
 
-        self.decoder.read(len, |keys| {
-            for k in keys {
-                let view = dict
+        let mut error = None;
+        let read = self.decoder.read(len, |keys| {
+            if base_buffer_idx == 0 {
+                // the dictionary buffers are the last buffers in output, we can directly use the views
+                output
                     .views
-                    .get(*k as usize)
-                    .ok_or_else(|| general_err!("invalid key={} for dictionary", *k))?;
-                let len = *view as u32;
-                if len <= 12 {
-                    // directly append the view if it is inlined
-                    // Safety: the view is from the dictionary, so it is valid
-                    unsafe {
-                        output.append_raw_view_unchecked(view);
-                    }
-                } else {
-                    // correct the buffer index and append the view
-                    let mut view = ByteView::from(*view);
-                    view.buffer_index += base_buffer_idx;
-                    // Safety: the view is from the dictionary,
-                    // we corrected the index value to point it to output buffer, so it is valid
-                    unsafe {
-                        output.append_raw_view_unchecked(&view.into());
-                    }
-                }
+                    .extend(keys.iter().map(|k| match dict.views.get(*k as usize) {
+                        Some(&view) => view,
+                        None => {
+                            if error.is_none() {
+                                error = Some(general_err!("invalid key={} for dictionary", *k));
+                            }
+                            0
+                        }
+                    }));
+                Ok(())
+            } else {
+                output
+                    .views
+                    .extend(keys.iter().map(|k| match dict.views.get(*k as usize) {
+                        Some(&view) => {
+                            let len = view as u32;
+                            if len <= 12 {
+                                view
+                            } else {
+                                let mut view = ByteView::from(view);
+                                view.buffer_index += base_buffer_idx;
+                                view.into()
+                            }
+                        }
+                        None => {
+                            if error.is_none() {
+                                error = Some(general_err!("invalid key={} for dictionary", *k));
+                            }
+                            0
+                        }
+                    }));
+                Ok(())
             }
-            Ok(())
-        })
+        })?;
+        if let Some(e) = error {
+            return Err(e);
+        }
+        Ok(read)
     }
 
     fn skip(&mut self, dict: &ViewBuffer, to_skip: usize) -> Result<usize> {
@@ -561,15 +610,19 @@ impl ByteViewArrayDecoderDeltaLength {
 
         let mut current_offset = self.data_offset;
         let initial_offset = current_offset;
-        for length in src_lengths {
-            // # Safety
-            // The length is from the delta length decoder, so it is valid
-            // The start_offset is calculated from the lengths, so it is valid
-            // `start_offset + length` is guaranteed to be within the bounds of `data`, as checked in `new`
-            unsafe { output.append_view_unchecked(block_id, current_offset as u32, *length as u32) }
 
-            current_offset += *length as usize;
-        }
+        output.views.extend(src_lengths.iter().map(|length| {
+            let len = *length as u32;
+            let start_offset = current_offset;
+            current_offset += len as usize;
+            // # Safety
+            // The length and offset are guaranteed valid by the entry check in `new`
+            make_view(
+                &self.data[start_offset..start_offset + len as usize],
+                block_id,
+                start_offset as u32,
+            )
+        }));
 
         // Delta length encoding has continuous strings, we can validate utf8 in one go
         if self.validate_utf8 {
@@ -638,7 +691,7 @@ impl ByteViewArrayDecoderDelta {
                 // The buffer_id is the last buffer in the output buffers
                 // The offset is calculated from the buffer, so it is valid
                 unsafe {
-                    output.append_raw_view_unchecked(&view);
+                    output.append_raw_view_unchecked(view);
                 }
                 Ok(())
             })?
@@ -663,7 +716,7 @@ impl ByteViewArrayDecoderDelta {
                 // The offset is calculated from the buffer, so it is valid
                 // Utf-8 validation is done later
                 unsafe {
-                    output.append_raw_view_unchecked(&view);
+                    output.append_raw_view_unchecked(view);
                 }
                 Ok(())
             })?;
