@@ -39,10 +39,12 @@ mod empty_array;
 mod fixed_len_byte_array;
 mod fixed_size_list_array;
 mod list_array;
+mod list_view_array;
 mod map_array;
 mod null_array;
 mod primitive_array;
 mod row_group_cache;
+mod row_group_index;
 mod row_number;
 mod struct_array;
 
@@ -60,6 +62,7 @@ pub use byte_view_array::make_byte_view_array_reader;
 pub use fixed_len_byte_array::make_fixed_len_byte_array_reader;
 pub use fixed_size_list_array::FixedSizeListArrayReader;
 pub use list_array::ListArrayReader;
+pub use list_view_array::ListViewArrayReader;
 pub use map_array::MapArrayReader;
 pub use null_array::NullArrayReader;
 pub use primitive_array::PrimitiveArrayReader;
