@@ -21,20 +21,19 @@ use crate::arrow::schema::parquet_to_arrow_field;
 use crate::basic::Type as PhysicalType;
 use crate::column::page::PageIterator;
 use crate::data_type::{DataType, Int96};
-use crate::errors::{ParquetError, Result};
+use crate::errors::Result;
 use crate::schema::types::ColumnDescPtr;
 use arrow_array::{
-    ArrayRef, BooleanArray, Decimal32Array, Decimal64Array, Decimal128Array, Decimal256Array,
-    Float32Array, Float64Array, Int8Array, Int16Array, Int32Array, Int64Array,
-    TimestampMicrosecondArray, TimestampMillisecondArray, TimestampNanosecondArray,
-    TimestampSecondArray, UInt8Array, UInt16Array, UInt32Array, UInt64Array,
-    builder::{
-        TimestampMicrosecondBufferBuilder, TimestampMillisecondBufferBuilder,
-        TimestampNanosecondBufferBuilder, TimestampSecondBufferBuilder,
-    },
+    Array, ArrayRef, BooleanArray, Date64Array, Decimal64Array, Decimal128Array, Decimal256Array,
+    Float32Array, Float64Array, Int8Array, Int16Array, Int32Array, Int64Array, PrimitiveArray,
+    UInt8Array, UInt16Array, builder::PrimitiveDictionaryBuilder, cast::AsArray, downcast_integer,
+    types::*,
 };
-use arrow_buffer::{BooleanBuffer, Buffer, i256};
-use arrow_data::ArrayDataBuilder;
+use arrow_array::{
+    TimestampMicrosecondArray, TimestampMillisecondArray, TimestampNanosecondArray,
+    TimestampSecondArray, UInt32Array, UInt64Array,
+};
+use arrow_buffer::{BooleanBuffer, Buffer, NullBuffer, ScalarBuffer, i256};
 use arrow_schema::{DataType as ArrowType, TimeUnit};
 use std::any::Any;
 use std::sync::Arc;
@@ -63,37 +62,23 @@ impl IntoBuffer for Vec<bool> {
 
 impl IntoBuffer for Vec<Int96> {
     fn into_buffer(self, target_type: &ArrowType) -> Buffer {
+        let mut builder = Vec::with_capacity(self.len());
         match target_type {
             ArrowType::Timestamp(TimeUnit::Second, _) => {
-                let mut builder = TimestampSecondBufferBuilder::new(self.len());
-                for v in self {
-                    builder.append(v.to_seconds())
-                }
-                builder.finish()
+                builder.extend(self.iter().map(|x| x.to_seconds()));
             }
             ArrowType::Timestamp(TimeUnit::Millisecond, _) => {
-                let mut builder = TimestampMillisecondBufferBuilder::new(self.len());
-                for v in self {
-                    builder.append(v.to_millis())
-                }
-                builder.finish()
+                builder.extend(self.iter().map(|x| x.to_millis()));
             }
             ArrowType::Timestamp(TimeUnit::Microsecond, _) => {
-                let mut builder = TimestampMicrosecondBufferBuilder::new(self.len());
-                for v in self {
-                    builder.append(v.to_micros())
-                }
-                builder.finish()
+                builder.extend(self.iter().map(|x| x.to_micros()));
             }
             ArrowType::Timestamp(TimeUnit::Nanosecond, _) => {
-                let mut builder = TimestampNanosecondBufferBuilder::new(self.len());
-                for v in self {
-                    builder.append(v.to_nanos())
-                }
-                builder.finish()
+                builder.extend(self.iter().map(|x| x.to_nanos()));
             }
             _ => unreachable!("Invalid target_type for Int96."),
         }
+        Buffer::from_vec(builder)
     }
 }
 
@@ -166,323 +151,52 @@ where
 
     fn consume_batch(&mut self) -> Result<ArrayRef> {
         let target_type = &self.data_type;
-        let arrow_data_type = match T::get_physical_type() {
-            PhysicalType::BOOLEAN => ArrowType::Boolean,
-            PhysicalType::INT32 => {
-                match target_type {
-                    ArrowType::UInt32 => {
-                        // follow C++ implementation and use overflow/reinterpret cast from  i32 to u32 which will map
-                        // `i32::MIN..0` to `(i32::MAX as u32)..u32::MAX`
-                        ArrowType::UInt32
-                    }
-                    ArrowType::Decimal32(_, _) => target_type.clone(),
-                    _ => ArrowType::Int32,
-                }
-            }
-            PhysicalType::INT64 => {
-                match target_type {
-                    ArrowType::UInt64 => {
-                        // follow C++ implementation and use overflow/reinterpret cast from  i64 to u64 which will map
-                        // `i64::MIN..0` to `(i64::MAX as u64)..u64::MAX`
-                        ArrowType::UInt64
-                    }
-                    ArrowType::Decimal64(_, _) => target_type.clone(),
-                    _ => ArrowType::Int64,
-                }
-            }
-            PhysicalType::FLOAT => ArrowType::Float32,
-            PhysicalType::DOUBLE => ArrowType::Float64,
-            PhysicalType::INT96 => match target_type {
-                ArrowType::Timestamp(TimeUnit::Second, _) => target_type.clone(),
-                ArrowType::Timestamp(TimeUnit::Millisecond, _) => target_type.clone(),
-                ArrowType::Timestamp(TimeUnit::Microsecond, _) => target_type.clone(),
-                ArrowType::Timestamp(TimeUnit::Nanosecond, _) => target_type.clone(),
-                _ => unreachable!("INT96 must be a timestamp."),
-            },
-            PhysicalType::BYTE_ARRAY | PhysicalType::FIXED_LEN_BYTE_ARRAY => {
-                unreachable!("PrimitiveArrayReaders don't support complex physical types");
-            }
-        };
-
-        // Convert to arrays by using the Parquet physical type.
-        // The physical types are then cast to Arrow types if necessary
 
+        // Convert physical data to equivalent arrow type, and then perform
+        // coercion as needed
         let record_data = self
             .record_reader
             .consume_record_data()
             .into_buffer(target_type);
 
-        let array_data = ArrayDataBuilder::new(arrow_data_type)
-            .len(self.record_reader.num_values())
-            .add_buffer(record_data)
-            .null_bit_buffer(self.record_reader.consume_bitmap_buffer());
+        let len = self.record_reader.num_values();
+        let nulls = self
+            .record_reader
+            .consume_bitmap_buffer()
+            .map(|b| NullBuffer::new(BooleanBuffer::new(b, 0, len)));
 
-        let array_data = unsafe { array_data.build_unchecked() };
         let array: ArrayRef = match T::get_physical_type() {
-            PhysicalType::BOOLEAN => Arc::new(BooleanArray::from(array_data)),
-            PhysicalType::INT32 => match array_data.data_type() {
-                ArrowType::UInt32 => Arc::new(UInt32Array::from(array_data)),
-                ArrowType::Int32 => Arc::new(Int32Array::from(array_data)),
-                ArrowType::Decimal32(_, _) => Arc::new(Decimal32Array::from(array_data)),
-                _ => unreachable!(),
-            },
-            PhysicalType::INT64 => match array_data.data_type() {
-                ArrowType::UInt64 => Arc::new(UInt64Array::from(array_data)),
-                ArrowType::Int64 => Arc::new(Int64Array::from(array_data)),
-                ArrowType::Decimal64(_, _) => Arc::new(Decimal64Array::from(array_data)),
-                _ => unreachable!(),
-            },
-            PhysicalType::FLOAT => Arc::new(Float32Array::from(array_data)),
-            PhysicalType::DOUBLE => Arc::new(Float64Array::from(array_data)),
-            PhysicalType::INT96 => match target_type {
-                ArrowType::Timestamp(TimeUnit::Second, _) => {
-                    Arc::new(TimestampSecondArray::from(array_data))
-                }
-                ArrowType::Timestamp(TimeUnit::Millisecond, _) => {
-                    Arc::new(TimestampMillisecondArray::from(array_data))
-                }
-                ArrowType::Timestamp(TimeUnit::Microsecond, _) => {
-                    Arc::new(TimestampMicrosecondArray::from(array_data))
-                }
-                ArrowType::Timestamp(TimeUnit::Nanosecond, _) => {
-                    Arc::new(TimestampNanosecondArray::from(array_data))
-                }
-                _ => unreachable!("INT96 must be a timestamp."),
-            },
-
+            PhysicalType::BOOLEAN => Arc::new(BooleanArray::new(
+                BooleanBuffer::new(record_data, 0, len),
+                nulls,
+            )),
+            PhysicalType::INT32 => Arc::new(Int32Array::new(
+                ScalarBuffer::new(record_data, 0, len),
+                nulls,
+            )),
+            PhysicalType::INT64 => Arc::new(Int64Array::new(
+                ScalarBuffer::new(record_data, 0, len),
+                nulls,
+            )),
+            PhysicalType::FLOAT => Arc::new(Float32Array::new(
+                ScalarBuffer::new(record_data, 0, len),
+                nulls,
+            )),
+            PhysicalType::DOUBLE => Arc::new(Float64Array::new(
+                ScalarBuffer::new(record_data, 0, len),
+                nulls,
+            )),
+            PhysicalType::INT96 => Arc::new(Int64Array::new(
+                ScalarBuffer::new(record_data, 0, len),
+                nulls,
+            )),
             PhysicalType::BYTE_ARRAY | PhysicalType::FIXED_LEN_BYTE_ARRAY => {
                 unreachable!("PrimitiveArrayReaders don't support complex physical types");
             }
         };
 
-        // cast to Arrow type
-        // We make a strong assumption here that the casts should be infallible.
-        // If the cast fails because of incompatible datatypes, then there might
-        // be a bigger problem with how Arrow schemas are converted to Parquet.
-        //
-        // As there is not always a 1:1 mapping between Arrow and Parquet, there
-        // are datatypes which we must convert explicitly.
-        // These are:
-        // - date64: cast int32 to date32, then date32 to date64.
-        // - decimal: cast int32 to decimal, int64 to decimal
-        let array = match target_type {
-            // Using `arrow_cast::cast` has been found to be very slow for converting
-            // INT32 physical type to lower bitwidth logical types. Since rust casts
-            // are infallible, instead use `unary` which is much faster (by up to 40%).
-            // One consequence of this approach is that some malformed integer columns
-            // will return (an arguably correct) result rather than null.
-            // See https://github.com/apache/arrow-rs/issues/7040 for a discussion of this
-            // issue.
-            ArrowType::UInt8 if *(array.data_type()) == ArrowType::Int32 => {
-                let array = array
-                    .as_any()
-                    .downcast_ref::<Int32Array>()
-                    .unwrap()
-                    .unary(|i| i as u8) as UInt8Array;
-                Arc::new(array) as ArrayRef
-            }
-            ArrowType::Int8 if *(array.data_type()) == ArrowType::Int32 => {
-                let array = array
-                    .as_any()
-                    .downcast_ref::<Int32Array>()
-                    .unwrap()
-                    .unary(|i| i as i8) as Int8Array;
-                Arc::new(array) as ArrayRef
-            }
-            ArrowType::UInt16 if *(array.data_type()) == ArrowType::Int32 => {
-                let array = array
-                    .as_any()
-                    .downcast_ref::<Int32Array>()
-                    .unwrap()
-                    .unary(|i| i as u16) as UInt16Array;
-                Arc::new(array) as ArrayRef
-            }
-            ArrowType::Int16 if *(array.data_type()) == ArrowType::Int32 => {
-                let array = array
-                    .as_any()
-                    .downcast_ref::<Int32Array>()
-                    .unwrap()
-                    .unary(|i| i as i16) as Int16Array;
-                Arc::new(array) as ArrayRef
-            }
-            ArrowType::Date64 if *(array.data_type()) == ArrowType::Int32 => {
-                // this is cheap as it internally reinterprets the data
-                let a = arrow_cast::cast(&array, &ArrowType::Date32)?;
-                arrow_cast::cast(&a, target_type)?
-            }
-            ArrowType::Decimal64(p, s) if *(array.data_type()) == ArrowType::Int32 => {
-                // Apply conversion to all elements regardless of null slots as the conversion
-                // to `i64` is infallible. This improves performance by avoiding a branch in
-                // the inner loop (see docs for `PrimitiveArray::unary`).
-                let array = match array.data_type() {
-                    ArrowType::Int32 => array
-                        .as_any()
-                        .downcast_ref::<Int32Array>()
-                        .unwrap()
-                        .unary(|i| i as i64)
-                        as Decimal64Array,
-                    _ => {
-                        return Err(arrow_err!(
-                            "Cannot convert {:?} to decimal",
-                            array.data_type()
-                        ));
-                    }
-                }
-                .with_precision_and_scale(*p, *s)?;
-
-                Arc::new(array) as ArrayRef
-            }
-            ArrowType::Decimal128(p, s) => {
-                // See above comment. Conversion to `i128` is likewise infallible.
-                let array = match array.data_type() {
-                    ArrowType::Int32 => array
-                        .as_any()
-                        .downcast_ref::<Int32Array>()
-                        .unwrap()
-                        .unary(|i| i as i128)
-                        as Decimal128Array,
-                    ArrowType::Int64 => array
-                        .as_any()
-                        .downcast_ref::<Int64Array>()
-                        .unwrap()
-                        .unary(|i| i as i128)
-                        as Decimal128Array,
-                    _ => {
-                        return Err(arrow_err!(
-                            "Cannot convert {:?} to decimal",
-                            array.data_type()
-                        ));
-                    }
-                }
-                .with_precision_and_scale(*p, *s)?;
-
-                Arc::new(array) as ArrayRef
-            }
-            ArrowType::Decimal256(p, s) => {
-                // See above comment. Conversion to `i256` is likewise infallible.
-                let array = match array.data_type() {
-                    ArrowType::Int32 => array
-                        .as_any()
-                        .downcast_ref::<Int32Array>()
-                        .unwrap()
-                        .unary(|i| i256::from_i128(i as i128))
-                        as Decimal256Array,
-                    ArrowType::Int64 => array
-                        .as_any()
-                        .downcast_ref::<Int64Array>()
-                        .unwrap()
-                        .unary(|i| i256::from_i128(i as i128))
-                        as Decimal256Array,
-                    _ => {
-                        return Err(arrow_err!(
-                            "Cannot convert {:?} to decimal",
-                            array.data_type()
-                        ));
-                    }
-                }
-                .with_precision_and_scale(*p, *s)?;
-
-                Arc::new(array) as ArrayRef
-            }
-            ArrowType::Dictionary(_, value_type) => match value_type.as_ref() {
-                ArrowType::Decimal32(p, s) => {
-                    let array = match array.data_type() {
-                        ArrowType::Int32 => array
-                            .as_any()
-                            .downcast_ref::<Int32Array>()
-                            .unwrap()
-                            .unary(|i| i)
-                            as Decimal32Array,
-                        _ => {
-                            return Err(arrow_err!(
-                                "Cannot convert {:?} to decimal dictionary",
-                                array.data_type()
-                            ));
-                        }
-                    }
-                    .with_precision_and_scale(*p, *s)?;
-
-                    arrow_cast::cast(&array, target_type)?
-                }
-                ArrowType::Decimal64(p, s) => {
-                    let array = match array.data_type() {
-                        ArrowType::Int32 => array
-                            .as_any()
-                            .downcast_ref::<Int32Array>()
-                            .unwrap()
-                            .unary(|i| i as i64)
-                            as Decimal64Array,
-                        ArrowType::Int64 => array
-                            .as_any()
-                            .downcast_ref::<Int64Array>()
-                            .unwrap()
-                            .unary(|i| i)
-                            as Decimal64Array,
-                        _ => {
-                            return Err(arrow_err!(
-                                "Cannot convert {:?} to decimal dictionary",
-                                array.data_type()
-                            ));
-                        }
-                    }
-                    .with_precision_and_scale(*p, *s)?;
-
-                    arrow_cast::cast(&array, target_type)?
-                }
-                ArrowType::Decimal128(p, s) => {
-                    let array = match array.data_type() {
-                        ArrowType::Int32 => array
-                            .as_any()
-                            .downcast_ref::<Int32Array>()
-                            .unwrap()
-                            .unary(|i| i as i128)
-                            as Decimal128Array,
-                        ArrowType::Int64 => array
-                            .as_any()
-                            .downcast_ref::<Int64Array>()
-                            .unwrap()
-                            .unary(|i| i as i128)
-                            as Decimal128Array,
-                        _ => {
-                            return Err(arrow_err!(
-                                "Cannot convert {:?} to decimal dictionary",
-                                array.data_type()
-                            ));
-                        }
-                    }
-                    .with_precision_and_scale(*p, *s)?;
-
-                    arrow_cast::cast(&array, target_type)?
-                }
-                ArrowType::Decimal256(p, s) => {
-                    let array = match array.data_type() {
-                        ArrowType::Int32 => array
-                            .as_any()
-                            .downcast_ref::<Int32Array>()
-                            .unwrap()
-                            .unary(i256::from)
-                            as Decimal256Array,
-                        ArrowType::Int64 => array
-                            .as_any()
-                            .downcast_ref::<Int64Array>()
-                            .unwrap()
-                            .unary(i256::from)
-                            as Decimal256Array,
-                        _ => {
-                            return Err(arrow_err!(
-                                "Cannot convert {:?} to decimal dictionary",
-                                array.data_type()
-                            ));
-                        }
-                    }
-                    .with_precision_and_scale(*p, *s)?;
-
-                    arrow_cast::cast(&array, target_type)?
-                }
-                _ => arrow_cast::cast(&array, target_type)?,
-            },
-            _ => arrow_cast::cast(&array, target_type)?,
-        };
+        // Coerce the arrow type to the desired array type
+        let array = coerce_array(array, target_type)?;
 
         // save definition and repetition buffers
         self.def_levels_buffer = self.record_reader.consume_def_levels();
@@ -504,6 +218,220 @@ where
     }
 }
 
+/// Coerce the parquet physical type array to the target type
+///
+/// This should match the logic in schema::primitive::apply_hint
+fn coerce_array(array: ArrayRef, target_type: &ArrowType) -> Result<ArrayRef> {
+    if let ArrowType::Dictionary(key_type, value_type) = target_type {
+        let dictionary = pack_dictionary(key_type, array.as_ref())?;
+        let any_dictionary = dictionary.as_any_dictionary();
+
+        let coerced_values =
+            coerce_array(Arc::clone(any_dictionary.values()), value_type.as_ref())?;
+
+        return Ok(any_dictionary.with_values(coerced_values));
+    }
+
+    match array.data_type() {
+        ArrowType::Int32 => coerce_i32(array.as_primitive(), target_type),
+        ArrowType::Int64 => coerce_i64(array.as_primitive(), target_type),
+        ArrowType::Boolean | ArrowType::Float32 | ArrowType::Float64 => Ok(array),
+        _ => unreachable!("Cannot coerce array of type {}", array.data_type()),
+    }
+}
+
+fn coerce_i32(array: &Int32Array, target_type: &ArrowType) -> Result<ArrayRef> {
+    Ok(match target_type {
+        ArrowType::UInt8 => {
+            let array = array.unary(|i| i as u8) as UInt8Array;
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::Int8 => {
+            let array = array.unary(|i| i as i8) as Int8Array;
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::UInt16 => {
+            let array = array.unary(|i| i as u16) as UInt16Array;
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::Int16 => {
+            let array = array.unary(|i| i as i16) as Int16Array;
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::Int32 => Arc::new(array.clone()),
+        // follow C++ implementation and use overflow/reinterpret cast from  i32 to u32 which will map
+        // `i32::MIN..0` to `(i32::MAX as u32)..u32::MAX`
+        ArrowType::UInt32 => Arc::new(UInt32Array::new(
+            array.values().inner().clone().into(),
+            array.nulls().cloned(),
+        )) as ArrayRef,
+        ArrowType::Date32 => Arc::new(array.reinterpret_cast::<Date32Type>()) as _,
+        ArrowType::Date64 => {
+            let array: Date64Array = array.unary(|x| x as i64 * 86_400_000);
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::Time32(TimeUnit::Second) => {
+            Arc::new(array.reinterpret_cast::<Time32SecondType>()) as ArrayRef
+        }
+        ArrowType::Time32(TimeUnit::Millisecond) => {
+            Arc::new(array.reinterpret_cast::<Time32MillisecondType>()) as ArrayRef
+        }
+        ArrowType::Timestamp(time_unit, timezone) => match time_unit {
+            TimeUnit::Second => {
+                let array: TimestampSecondArray = array
+                    .unary(|x| x as i64)
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+            TimeUnit::Millisecond => {
+                let array: TimestampMillisecondArray = array
+                    .unary(|x| x as i64)
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+            TimeUnit::Microsecond => {
+                let array: TimestampMicrosecondArray = array
+                    .unary(|x| x as i64)
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+            TimeUnit::Nanosecond => {
+                let array: TimestampNanosecondArray = array
+                    .unary(|x| x as i64)
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+        },
+        ArrowType::Decimal32(p, s) => {
+            let array = array
+                .reinterpret_cast::<Decimal32Type>()
+                .with_precision_and_scale(*p, *s)?;
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::Decimal64(p, s) => {
+            let array: Decimal64Array =
+                array.unary(|i| i as i64).with_precision_and_scale(*p, *s)?;
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::Decimal128(p, s) => {
+            let array: Decimal128Array = array
+                .unary(|i| i as i128)
+                .with_precision_and_scale(*p, *s)?;
+            Arc::new(array) as ArrayRef
+        }
+        ArrowType::Decimal256(p, s) => {
+            let array: Decimal256Array = array
+                .unary(|i| i256::from_i128(i as i128))
+                .with_precision_and_scale(*p, *s)?;
+            Arc::new(array) as ArrayRef
+        }
+        _ => unreachable!("Cannot coerce i32 to {target_type}"),
+    })
+}
+
+fn coerce_i64(array: &Int64Array, target_type: &ArrowType) -> Result<ArrayRef> {
+    Ok(match target_type {
+        ArrowType::Int64 => Arc::new(array.clone()) as _,
+        // follow C++ implementation and use overflow/reinterpret cast from i64 to u64 which will map
+        // `i64::MIN..0` to `(i64::MAX as u64)..u64::MAX`
+        ArrowType::UInt64 => Arc::new(UInt64Array::new(
+            array.values().inner().clone().into(),
+            array.nulls().cloned(),
+        )) as ArrayRef,
+        ArrowType::Date64 => Arc::new(array.reinterpret_cast::<Date64Type>()) as _,
+        ArrowType::Time64(TimeUnit::Microsecond) => {
+            Arc::new(array.reinterpret_cast::<Time64MicrosecondType>()) as _
+        }
+        ArrowType::Time64(TimeUnit::Nanosecond) => {
+            Arc::new(array.reinterpret_cast::<Time64NanosecondType>()) as _
+        }
+        ArrowType::Duration(unit) => match unit {
+            TimeUnit::Second => Arc::new(array.reinterpret_cast::<DurationSecondType>()) as _,
+            TimeUnit::Millisecond => {
+                Arc::new(array.reinterpret_cast::<DurationMillisecondType>()) as _
+            }
+            TimeUnit::Microsecond => {
+                Arc::new(array.reinterpret_cast::<DurationMicrosecondType>()) as _
+            }
+            TimeUnit::Nanosecond => {
+                Arc::new(array.reinterpret_cast::<DurationNanosecondType>()) as _
+            }
+        },
+        ArrowType::Timestamp(time_unit, timezone) => match time_unit {
+            TimeUnit::Second => {
+                let array = array
+                    .reinterpret_cast::<TimestampSecondType>()
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+            TimeUnit::Millisecond => {
+                let array = array
+                    .reinterpret_cast::<TimestampMillisecondType>()
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+            TimeUnit::Microsecond => {
+                let array = array
+                    .reinterpret_cast::<TimestampMicrosecondType>()
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+            TimeUnit::Nanosecond => {
+                let array = array
+                    .reinterpret_cast::<TimestampNanosecondType>()
+                    .with_timezone_opt(timezone.clone());
+                Arc::new(array) as _
+            }
+        },
+        ArrowType::Decimal64(p, s) => {
+            let array = array
+                .reinterpret_cast::<Decimal64Type>()
+                .with_precision_and_scale(*p, *s)?;
+            Arc::new(array) as _
+        }
+        ArrowType::Decimal128(p, s) => {
+            let array: Decimal128Array = array
+                .unary(|i| i as i128)
+                .with_precision_and_scale(*p, *s)?;
+            Arc::new(array) as _
+        }
+        ArrowType::Decimal256(p, s) => {
+            let array: Decimal256Array = array
+                .unary(|i| i256::from_i128(i as i128))
+                .with_precision_and_scale(*p, *s)?;
+            Arc::new(array) as _
+        }
+        _ => unreachable!("Cannot coerce i64 to {target_type}"),
+    })
+}
+
+macro_rules! pack_dictionary_helper {
+    ($t:ty, $values:ident) => {
+        match $values.data_type() {
+            ArrowType::Int32 => pack_dictionary_impl::<$t, Int32Type>($values.as_primitive()),
+            ArrowType::Int64 => pack_dictionary_impl::<$t, Int64Type>($values.as_primitive()),
+            ArrowType::Float32 => pack_dictionary_impl::<$t, Float32Type>($values.as_primitive()),
+            ArrowType::Float64 => pack_dictionary_impl::<$t, Float64Type>($values.as_primitive()),
+            _ => unreachable!("Invalid physical type"),
+        }
+    };
+}
+
+fn pack_dictionary(key: &ArrowType, values: &dyn Array) -> Result<ArrayRef> {
+    downcast_integer! {
+        key => (pack_dictionary_helper, values),
+        _ => unreachable!("Invalid key type"),
+    }
+}
+
+fn pack_dictionary_impl<K: ArrowDictionaryKeyType, V: ArrowPrimitiveType>(
+    values: &PrimitiveArray<V>,
+) -> Result<ArrayRef> {
+    let mut builder = PrimitiveDictionaryBuilder::<K, V>::with_capacity(1024, values.len());
+    builder.extend(values);
+    Ok(Arc::new(builder.finish()))
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
