@@ -18,7 +18,14 @@
 use crate::arrow::buffer::offset_buffer::OffsetBuffer;
 use crate::arrow::record_reader::buffer::ValuesBuffer;
 use crate::errors::{ParquetError, Result};
-use arrow_array::{Array, ArrayRef, OffsetSizeTrait, make_array};
+use arrow_array::{Array, GenericByteArray, downcast_integer};
+use arrow_array::{
+    ArrayRef, FixedSizeBinaryArray, OffsetSizeTrait,
+    builder::{FixedSizeBinaryDictionaryBuilder, GenericByteDictionaryBuilder},
+    cast::AsArray,
+    make_array,
+    types::{ArrowDictionaryKeyType, ByteArrayType},
+};
 use arrow_buffer::{ArrowNativeType, Buffer};
 use arrow_data::ArrayDataBuilder;
 use arrow_schema::DataType as ArrowType;
@@ -158,7 +165,12 @@ impl<K: ArrowNativeType + Ord, V: OffsetSizeTrait> DictionaryBuffer<K, V> {
                     unreachable!()
                 };
                 let values = if let ArrowType::FixedSizeBinary(size) = **value_type {
-                    arrow_cast::cast(&values, &ArrowType::FixedSizeBinary(size)).unwrap()
+                    let binary = values.as_binary::<i32>();
+                    Arc::new(FixedSizeBinaryArray::new(
+                        size,
+                        binary.values().clone(),
+                        binary.nulls().cloned(),
+                    )) as _
                 } else {
                     values
                 };
@@ -177,17 +189,13 @@ impl<K: ArrowNativeType + Ord, V: OffsetSizeTrait> DictionaryBuffer<K, V> {
                 Ok(make_array(data))
             }
             Self::Values { values } => {
-                let value_type = match data_type {
-                    ArrowType::Dictionary(_, v) => v.as_ref().clone(),
+                let (key_type, value_type) = match data_type {
+                    ArrowType::Dictionary(k, v) => (k, v.as_ref().clone()),
                     _ => unreachable!(),
                 };
 
-                // This will compute a new dictionary
-                let array =
-                    arrow_cast::cast(&values.into_array(null_buffer, value_type), data_type)
-                        .expect("cast should be infallible");
-
-                Ok(array)
+                let array = values.into_array(null_buffer, value_type);
+                pack_values(key_type, &array)
             }
         }
     }
@@ -213,6 +221,60 @@ impl<K: ArrowNativeType, V: OffsetSizeTrait> ValuesBuffer for DictionaryBuffer<K
     }
 }
 
+macro_rules! dict_helper {
+    ($k:ty, $array:ident) => {
+        match $array.data_type() {
+            ArrowType::Utf8 => pack_values_impl::<$k, _>($array.as_string::<i32>()),
+            ArrowType::LargeUtf8 => pack_values_impl::<$k, _>($array.as_string::<i64>()),
+            ArrowType::Binary => pack_values_impl::<$k, _>($array.as_binary::<i32>()),
+            ArrowType::LargeBinary => pack_values_impl::<$k, _>($array.as_binary::<i64>()),
+            ArrowType::FixedSizeBinary(_) => {
+                pack_fixed_values_impl::<$k>($array.as_fixed_size_binary())
+            }
+            _ => unreachable!(),
+        }
+    };
+}
+
+fn pack_values(key_type: &ArrowType, values: &ArrayRef) -> Result<ArrayRef> {
+    downcast_integer! {
+        key_type => (dict_helper, values),
+            _ => unreachable!(),
+    }
+}
+
+fn pack_values_impl<K: ArrowDictionaryKeyType, T: ByteArrayType>(
+    array: &GenericByteArray<T>,
+) -> Result<ArrayRef> {
+    let mut builder = GenericByteDictionaryBuilder::<K, T>::with_capacity(array.len(), 1024, 1024);
+    for x in array {
+        match x {
+            Some(x) => builder.append_value(x),
+            None => builder.append_null(),
+        }
+    }
+    let raw = builder.finish();
+    Ok(Arc::new(raw))
+}
+
+fn pack_fixed_values_impl<K: ArrowDictionaryKeyType>(
+    array: &FixedSizeBinaryArray,
+) -> Result<ArrayRef> {
+    let mut builder = FixedSizeBinaryDictionaryBuilder::<K>::with_capacity(
+        array.len(),
+        1024,
+        array.value_length(),
+    );
+    for x in array {
+        match x {
+            Some(x) => builder.append_value(x),
+            None => builder.append_null(),
+        }
+    }
+    let raw = builder.finish();
+    Ok(Arc::new(raw))
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
