@@ -60,13 +60,17 @@ impl DictIndexDecoder {
     /// and calling `f` with each decoded dictionary index
     ///
     /// Will short-circuit and return on error
+    #[inline(always)]
     pub fn read<F: FnMut(&[i32]) -> Result<()>>(&mut self, len: usize, mut f: F) -> Result<usize> {
+        let total_to_read = len.min(self.max_remaining_values);
+
         let mut values_read = 0;
 
-        while values_read != len && self.max_remaining_values != 0 {
+        let index_buf = self.index_buf.as_mut();
+        while values_read < total_to_read {
             if self.index_offset == self.index_buf_len {
                 // We've consumed the entire index buffer so we need to reload it before proceeding
-                let read = self.decoder.get_batch(self.index_buf.as_mut())?;
+                let read = self.decoder.get_batch(index_buf)?;
                 if read == 0 {
                     break;
                 }
@@ -74,16 +78,16 @@ impl DictIndexDecoder {
                 self.index_offset = 0;
             }
 
-            let to_read = (len - values_read)
-                .min(self.index_buf_len - self.index_offset)
-                .min(self.max_remaining_values);
+            let available = self.index_buf_len - self.index_offset;
+            let n = available.min(total_to_read - values_read);
 
-            f(&self.index_buf[self.index_offset..self.index_offset + to_read])?;
+            f(&index_buf[self.index_offset..self.index_offset + n])?;
 
-            self.index_offset += to_read;
-            values_read += to_read;
-            self.max_remaining_values -= to_read;
+            self.index_offset += n;
+            values_read += n;
         }
+        self.max_remaining_values -= values_read;
+
         Ok(values_read)
     }
 
