@@ -20,7 +20,7 @@ use std::sync::Arc;
 
 use crate::arrow::schema::extension::try_add_extension_type;
 use crate::arrow::schema::primitive::convert_primitive;
-use crate::arrow::schema::virtual_type::RowNumber;
+use crate::arrow::schema::virtual_type::{RowGroupIndex, RowNumber};
 use crate::arrow::{PARQUET_FIELD_ID_META_KEY, ProjectionMask};
 use crate::basic::{ConvertedType, Repetition};
 use crate::errors::ParquetError;
@@ -88,6 +88,8 @@ impl ParquetField {
 pub enum VirtualColumnType {
     /// Row number within the file
     RowNumber,
+    /// Row group index
+    RowGroupIndex,
 }
 
 #[derive(Debug, Clone)]
@@ -434,6 +436,8 @@ impl Visitor {
             Some(DataType::List(f)) => Some(f.as_ref()),
             Some(DataType::LargeList(f)) => Some(f.as_ref()),
             Some(DataType::FixedSizeList(f, _)) => Some(f.as_ref()),
+            Some(DataType::ListView(f)) => Some(f.as_ref()),
+            Some(DataType::LargeListView(f)) => Some(f.as_ref()),
             Some(d) => {
                 return Err(arrow_err!(
                     "incompatible arrow schema, expected list got {}",
@@ -517,6 +521,8 @@ impl Visitor {
                     Some(DataType::FixedSizeList(_, len)) => {
                         DataType::FixedSizeList(item_field, len)
                     }
+                    Some(DataType::ListView(_)) => DataType::ListView(item_field),
+                    Some(DataType::LargeListView(_)) => DataType::LargeListView(item_field),
                     _ => DataType::List(item_field),
                 };
 
@@ -584,6 +590,7 @@ pub(super) fn convert_virtual_field(
 
     let virtual_type = match extension_name {
         RowNumber::NAME => VirtualColumnType::RowNumber,
+        RowGroupIndex::NAME => VirtualColumnType::RowGroupIndex,
         _ => {
             return Err(ParquetError::ArrowError(format!(
                 "unsupported virtual column type '{}' for field '{}'",
