@@ -503,20 +503,27 @@ pub use lz4_codec::*;
 
 #[cfg(any(feature = "zstd", test))]
 mod zstd_codec {
-    use std::io::{self, Write};
-
     use crate::compression::{Codec, ZstdLevel};
     use crate::errors::Result;
 
     /// Codec for Zstandard compression algorithm.
+    ///
+    /// Uses `zstd::bulk` API with reusable compressor/decompressor contexts
+    /// to avoid the overhead of reinitializing contexts for each operation.
     pub struct ZSTDCodec {
-        level: ZstdLevel,
+        compressor: zstd::bulk::Compressor<'static>,
+        decompressor: zstd::bulk::Decompressor<'static>,
     }
 
     impl ZSTDCodec {
         /// Creates new Zstandard compression codec.
         pub(crate) fn new(level: ZstdLevel) -> Self {
-            Self { level }
+            Self {
+                compressor: zstd::bulk::Compressor::new(level.compression_level())
+                    .expect("valid zstd compression level"),
+                decompressor: zstd::bulk::Decompressor::new()
+                    .expect("can create zstd decompressor"),
+            }
         }
     }
 
@@ -525,22 +532,25 @@ mod zstd_codec {
             &mut self,
             input_buf: &[u8],
             output_buf: &mut Vec<u8>,
-            _uncompress_size: Option<usize>,
+            uncompress_size: Option<usize>,
         ) -> Result<usize> {
-            let mut decoder = zstd::Decoder::new(input_buf)?;
-            match io::copy(&mut decoder, output_buf) {
-                Ok(n) => Ok(n as usize),
-                Err(e) => Err(e.into()),
-            }
+            let capacity = uncompress_size.unwrap_or_else(|| {
+                // Get the decompressed size from the zstd frame header
+                zstd::zstd_safe::get_frame_content_size(input_buf)
+                    .ok()
+                    .flatten()
+                    .unwrap_or(input_buf.len() as u64 * 4) as usize
+            });
+            let decompressed = self.decompressor.decompress(input_buf, capacity)?;
+            let len = decompressed.len();
+            output_buf.extend_from_slice(&decompressed);
+            Ok(len)
         }
 
         fn compress(&mut self, input_buf: &[u8], output_buf: &mut Vec<u8>) -> Result<()> {
-            let mut encoder = zstd::Encoder::new(output_buf, self.level.0)?;
-            encoder.write_all(input_buf)?;
-            match encoder.finish() {
-                Ok(_) => Ok(()),
-                Err(e) => Err(e.into()),
-            }
+            let compressed = self.compressor.compress(input_buf)?;
+            output_buf.extend_from_slice(&compressed);
+            Ok(())
         }
     }
 }
