@@ -198,8 +198,8 @@ mod tests {
         assert_eq!(field.name(), "data");
         // data should have been written with the Variant logical type
         assert_eq!(
-            field.get_basic_info().logical_type(),
-            Some(crate::basic::LogicalType::Variant {
+            field.get_basic_info().logical_type_ref(),
+            Some(&crate::basic::LogicalType::Variant {
                 specification_version: None
             })
         );
