@@ -34,7 +34,7 @@ use parquet::data_type::{ByteArray, ByteArrayType};
 use parquet::encryption::decrypt::FileDecryptionProperties;
 use parquet::encryption::encrypt::FileEncryptionProperties;
 use parquet::errors::ParquetError;
-use parquet::file::metadata::{ColumnChunkMetaData, ParquetMetaData};
+use parquet::file::metadata::{ColumnChunkMetaData, PageIndexPolicy, ParquetMetaData};
 use parquet::file::properties::WriterProperties;
 use parquet::file::writer::SerializedFileWriter;
 use parquet::schema::parser::parse_message_type;
@@ -432,7 +432,7 @@ fn uniform_encryption_roundtrip(
 
     let props = WriterProperties::builder()
         // Ensure multiple row groups
-        .set_max_row_group_size(50)
+        .set_max_row_group_row_count(Some(50))
         // Ensure multiple pages per row group
         .set_write_batch_size(20)
         .set_data_page_row_count_limit(20)
@@ -453,7 +453,7 @@ fn uniform_encryption_roundtrip(
 
     let options = ArrowReaderOptions::new()
         .with_file_decryption_properties(decryption_properties)
-        .with_page_index(page_index);
+        .with_page_index_policy(PageIndexPolicy::from(page_index));
 
     let builder = ParquetRecordBatchReaderBuilder::try_new_with_options(file, options)?;
     assert_eq!(&row_group_sizes(builder.metadata()), &[50, 50, 50]);
@@ -537,7 +537,7 @@ fn uniform_encryption_page_skipping(page_index: bool) -> parquet::errors::Result
 
     let props = WriterProperties::builder()
         // Ensure multiple row groups
-        .set_max_row_group_size(50)
+        .set_max_row_group_row_count(Some(50))
         // Ensure multiple pages per row group
         .set_write_batch_size(20)
         .set_data_page_row_count_limit(20)
@@ -557,7 +557,7 @@ fn uniform_encryption_page_skipping(page_index: bool) -> parquet::errors::Result
 
     let options = ArrowReaderOptions::new()
         .with_file_decryption_properties(decryption_properties)
-        .with_page_index(page_index);
+        .with_page_index_policy(PageIndexPolicy::from(page_index));
 
     let builder = ParquetRecordBatchReaderBuilder::try_new_with_options(file, options)?;
 
@@ -1254,7 +1254,7 @@ fn test_decrypt_page_index(
     let file = File::open(path)?;
     let options = ArrowReaderOptions::default()
         .with_file_decryption_properties(decryption_properties)
-        .with_page_index(true);
+        .with_page_index_policy(PageIndexPolicy::from(true));
 
     let arrow_metadata = ArrowReaderMetadata::load(&file, options)?;
 
